@@ -1,5 +1,5 @@
 /*
- * $Id: directory.c,v 1.71.2.4.2.15.2.1 2004/10/20 21:43:33 didg Exp $
+ * $Id: directory.c,v 1.71.2.4.2.15.2.5 2005/02/10 01:23:10 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -786,7 +786,7 @@ static int copydir(const struct vol *vol, char *src, char *dst)
             if (S_ISDIR(st.st_mode)) {
                 if (AFP_OK != (err = copydir(vol, spath, dpath)))
                     goto copydir_done;
-            } else if (AFP_OK != (err = copyfile(vol, vol, spath, dpath, NULL))) {
+            } else if (AFP_OK != (err = copyfile(vol, vol, spath, dpath, NULL, NULL))) {
                 goto copydir_done;
 
             } else {
@@ -979,6 +979,7 @@ char	**cpath;
     u_int16_t   len16;
     int         size = 0;
     char        sep;
+    int         toUTF8 = 0;
        	
     data = *cpath;
     afp_errno = AFPERR_NOOBJ;
@@ -988,6 +989,10 @@ char	**cpath;
        len = (unsigned char) *data++;
        size = 2;
        sep = 0;
+       if (afp_version >= 30) {
+           ret.m_type = 3;
+           toUTF8 = 1;
+       }
        break;
     case 3:
        if (afp_version >= 30) {
@@ -1104,9 +1109,20 @@ char	**cpath;
         if ( p != path ) { /* we got something */
             ret.u_name = NULL;
             if (afp_version >= 30) {
-                /* check for OS X mangled filename :( */
                 char *t;
                 cnid_t fileid;
+                
+                if (toUTF8) {
+                    static char	temp[ MAXPATHLEN + 1];
+
+                    /* not an UTF8 name */
+                    if (mtoUTF8(vol, path, strlen(path), temp, MAXPATHLEN) == -1) {
+                        afp_errno = AFPERR_PARAM;
+                        return( NULL );
+                    }
+                    strcpy(path, temp);
+                }
+                /* check for OS X mangled filename :( */
 	    
                 t = demangle_osx(vol, path, dir->d_did, &fileid);
                 if (t != path) {
@@ -1302,7 +1318,7 @@ int getdirparams(const struct vol *vol,
 		   (1 << DIRPBIT_BDATE) |
 		   (1 << DIRPBIT_FINFO)))) {
 
-        ad_init(&ad, vol->v_adouble);
+        ad_init(&ad, vol->v_adouble, vol->v_ad_options);
     	if ( !ad_metadata( upath, ADFLAGS_DIR, &ad) ) {
             isad = 1;
         }
@@ -1634,7 +1650,7 @@ int setdirparams(const struct vol *vol,
     u_int16_t           bitmap = d_bitmap;
     u_char              finder_buf[32];
     u_int32_t		upriv;
-    mode_t              mpriv;
+    mode_t              mpriv;          /* uninitialized, OK 310105 */
     u_int16_t           upriv_bit = 0;
 
     bit = 0;
@@ -1740,7 +1756,7 @@ int setdirparams(const struct vol *vol,
         bitmap = bitmap>>1;
         bit++;
     }
-    ad_init(&ad, vol->v_adouble);
+    ad_init(&ad, vol->v_adouble, vol->v_ad_options);
 
     if (ad_open( upath, vol_noadouble(vol)|ADFLAGS_HF|ADFLAGS_DIR,
                  O_RDWR|O_CREAT, 0666, &ad) < 0) {
@@ -1925,8 +1941,7 @@ setdirparam_done:
             struct stat *st = &path->st;
 
             if (dir && dir->d_parent) {
-                ad_setid(&ad,(vol->v_flags & AFPVOL_NODEV)?0:st->st_dev, 
-                         st->st_ino,  dir->d_did, dir->d_parent->d_did, vol->v_stamp);
+                ad_setid(&ad, st->st_dev, st->st_ino,  dir->d_did, dir->d_parent->d_did, vol->v_stamp);
             }
         }
         ad_flush( &ad, ADFLAGS_HF );
@@ -2011,7 +2026,7 @@ int		ibuflen, *rbuflen;
         return( AFPERR_PARAM );
     }
 
-    ad_init(&ad, vol->v_adouble);
+    ad_init(&ad, vol->v_adouble, vol->v_ad_options);
     if (ad_open( ".", vol_noadouble(vol)|ADFLAGS_HF|ADFLAGS_DIR,
                  O_RDWR|O_CREAT, 0666, &ad ) < 0)  {
         if (vol_noadouble(vol))
@@ -2019,8 +2034,7 @@ int		ibuflen, *rbuflen;
         return( AFPERR_ACCESS );
     }
     ad_setname(&ad, s_path->m_name);
-    ad_setid( &ad, (vol->v_flags & AFPVOL_NODEV)?0:s_path->st.st_dev,
-                   s_path->st.st_ino, dir->d_did, did, vol->v_stamp);
+    ad_setid( &ad, s_path->st.st_dev, s_path->st.st_ino, dir->d_did, did, vol->v_stamp);
 
     ad_flush( &ad, ADFLAGS_HF );
     ad_close( &ad, ADFLAGS_HF );
@@ -2086,7 +2100,7 @@ struct dir	*dir, *newparent;
      * .Parent
     */
     
-    ad_init(&ad, vol->v_adouble);
+    ad_init(&ad, vol->v_adouble, vol->v_ad_options);
 
     if (!ad_open( dst, ADFLAGS_HF|ADFLAGS_DIR, O_RDWR, 0, &ad)) {
         ad_setname(&ad, newname);
@@ -2153,7 +2167,7 @@ int pathlen;
 
     fdir = curdir;
 
-    ad_init(&ad, vol->v_adouble);
+    ad_init(&ad, vol->v_adouble, vol->v_ad_options);
     if ( ad_metadata( ".", ADFLAGS_DIR, &ad) == 0 ) {
 
         ad_getattr(&ad, &ashort);
