@@ -1,5 +1,5 @@
 /*
- * $Id: main.c,v 1.20.2.1 2003/11/05 06:41:01 didg Exp $
+ * $Id: main.c,v 1.20.4.2.2.9 2004/05/12 21:21:48 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -12,16 +12,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
 #include <signal.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/param.h>
 #include <sys/uio.h>
 #include <atalk/logger.h>
@@ -30,17 +22,18 @@
 
 #include <errno.h>
 
-#include <netatalk/endian.h>
+#include <atalk/adouble.h>
+
 #include <netatalk/at.h>
 #include <atalk/compat.h>
 #include <atalk/dsi.h>
 #include <atalk/atp.h>
 #include <atalk/asp.h>
 #include <atalk/afp.h>
-#include <atalk/adouble.h>
 #include <atalk/paths.h>
 #include <atalk/util.h>
 #include <atalk/server_child.h>
+#include <atalk/server_ipc.h>
 
 #include "globals.h"
 #include "afp_config.h"
@@ -63,6 +56,7 @@ struct afp_options default_options;
 static AFPConfig *configs;
 static server_child *server_children;
 static fd_set save_rfds;
+static int    Ipc_fd = -1;
 
 #ifdef TRU64
 void afp_get_cmdline( int *ac, char ***av)
@@ -78,20 +72,43 @@ static void afp_exit(const int i)
     exit(i);
 }
 
+/* ------------------
+   initialize fd set we are waiting for.
+*/
+static void set_fd(int ipc_fd)
+{
+    AFPConfig   *config;
+
+    FD_ZERO(&save_rfds);
+    for (config = configs; config; config = config->next) {
+        if (config->fd < 0) /* for proxies */
+            continue;
+        FD_SET(config->fd, &save_rfds);
+    }
+    if (ipc_fd >= 0) {
+        FD_SET(ipc_fd, &save_rfds);
+    }
+}
+ 
+/* ------------------ */
 static void afp_goaway(int sig)
 {
+
 #ifndef NO_DDP
     asp_kill(sig);
 #endif /* ! NO_DDP */
+
     dsi_kill(sig);
     switch( sig ) {
     case SIGTERM :
         LOG(log_info, logtype_afpd, "shutting down on signal %d", sig );
         break;
+    case SIGUSR1 :
     case SIGHUP :
         /* w/ a configuration file, we can force a re-read if we want */
         nologin++;
-        if ((nologin + 1) & 1) {
+        auth_unload();
+        if (sig == SIGHUP || ((nologin + 1) & 1)) {
             AFPConfig *config;
 
             LOG(log_info, logtype_afpd, "re-reading configuration file");
@@ -99,20 +116,20 @@ static void afp_goaway(int sig)
                 if (config->server_cleanup)
                     config->server_cleanup(config);
 
+            /* configfree close atp socket used for DDP tickle, there's an issue
+             * with atp tid.
+            */
             configfree(configs, NULL);
             if (!(configs = configinit(&default_options))) {
                 LOG(log_error, logtype_afpd, "config re-read: no servers configured");
-                afp_exit(1);
-            }
-            FD_ZERO(&save_rfds);
-            for (config = configs; config; config = config->next) {
-                if (config->fd < 0)
-                    continue;
-                FD_SET(config->fd, &save_rfds);
+                afp_exit(EXITERR_CONF);
             }
+            set_fd(Ipc_fd);
         } else {
             LOG(log_info, logtype_afpd, "disallowing logins");
-            auth_unload();
+        }
+        if (sig == SIGHUP) {
+            nologin = 0;
         }
         break;
     default :
@@ -141,6 +158,7 @@ char	**av;
 {
     AFPConfig           *config;
     fd_set              rfds;
+    void                *ipc;
     struct sigaction	sv;
     sigset_t            sigs;
     int                 ret;
@@ -151,9 +169,12 @@ char	**av;
     set_auth_parameters( ac, av );
 #endif /* TRU64 */
 
+#ifdef DEBUG1
+    fault_setup(NULL);
+#endif
     afp_options_init(&default_options);
     if (!afp_options_parse(ac, av, &default_options))
-        exit(1);
+        exit(EXITERR_CONF);
 
     /* Save the user's current umask for use with CNID (and maybe some 
      * other things, too). */
@@ -162,53 +183,80 @@ char	**av;
     switch(server_lock("afpd", default_options.pidfile,
                        default_options.flags & OPTION_DEBUG)) {
     case -1: /* error */
-        exit(1);
+        exit(EXITERR_SYS);
     case 0: /* child */
         break;
     default: /* server */
         exit(0);
     }
 
+    /* Register CNID  */
+    cnid_init();
+
     /* install child handler for asp and dsi. we do this before afp_goaway
      * as afp_goaway references stuff from here. 
      * XXX: this should really be setup after the initial connections. */
     if (!(server_children = server_child_alloc(default_options.connections,
                             CHILD_NFORKS))) {
         LOG(log_error, logtype_afpd, "main: server_child alloc: %s", strerror(errno) );
-        afp_exit(1);
+        afp_exit(EXITERR_SYS);
     }
-
+    
+#ifdef AFP3x
+    /* linux at least up to 2.4.22 send a SIGXFZ for vfat fs,
+       even if the file is open with O_LARGEFILE ! */
+#ifdef SIGXFSZ
+    signal(SIGXFSZ , SIG_IGN); 
+#endif
+#endif    
+    
     memset(&sv, 0, sizeof(sv));
     sv.sa_handler = child_handler;
     sigemptyset( &sv.sa_mask );
     sigaddset(&sv.sa_mask, SIGALRM);
     sigaddset(&sv.sa_mask, SIGHUP);
     sigaddset(&sv.sa_mask, SIGTERM);
+    sigaddset(&sv.sa_mask, SIGUSR1);
+    
     sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGCHLD, &sv, 0 ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        afp_exit(1);
+        afp_exit(EXITERR_SYS);
     }
 
     sv.sa_handler = afp_goaway;
     sigemptyset( &sv.sa_mask );
     sigaddset(&sv.sa_mask, SIGALRM);
     sigaddset(&sv.sa_mask, SIGTERM);
+    sigaddset(&sv.sa_mask, SIGHUP);
     sigaddset(&sv.sa_mask, SIGCHLD);
     sv.sa_flags = SA_RESTART;
-    if ( sigaction( SIGHUP, &sv, 0 ) < 0 ) {
+    if ( sigaction( SIGUSR1, &sv, 0 ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        afp_exit(1);
+        afp_exit(EXITERR_SYS);
     }
 
+    sigemptyset( &sv.sa_mask );
+    sigaddset(&sv.sa_mask, SIGALRM);
+    sigaddset(&sv.sa_mask, SIGTERM);
+    sigaddset(&sv.sa_mask, SIGUSR1);
+    sigaddset(&sv.sa_mask, SIGCHLD);
+    sv.sa_flags = SA_RESTART;
+    if ( sigaction( SIGHUP, &sv, 0 ) < 0 ) {
+        LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
+        afp_exit(EXITERR_SYS);
+    }
+
+
     sigemptyset( &sv.sa_mask );
     sigaddset(&sv.sa_mask, SIGALRM);
     sigaddset(&sv.sa_mask, SIGHUP);
+    sigaddset(&sv.sa_mask, SIGUSR1);
     sigaddset(&sv.sa_mask, SIGCHLD);
     sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGTERM, &sv, 0 ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        afp_exit(1);
+        afp_exit(EXITERR_SYS);
     }
 
     /* afpd.conf: not in config file: lockfile, connections, configfile
@@ -222,25 +270,25 @@ char	**av;
     sigemptyset(&sigs);
     sigaddset(&sigs, SIGALRM);
     sigaddset(&sigs, SIGHUP);
-#if 0 
-    /* don't block SIGTERM */   
+    sigaddset(&sigs, SIGUSR1);
+#if 0
+    /* don't block SIGTERM */
     sigaddset(&sigs, SIGTERM);
-#endif    
+#endif
     sigaddset(&sigs, SIGCHLD);
+
     sigprocmask(SIG_BLOCK, &sigs, NULL);
     if (!(configs = configinit(&default_options))) {
-        LOG(log_error, logtype_afpd, "main: no servers configured: %s\n", strerror(errno));
-        afp_exit(1);
+        LOG(log_error, logtype_afpd, "main: no servers configured: %s", strerror(errno));
+        afp_exit(EXITERR_CONF);
     }
     sigprocmask(SIG_UNBLOCK, &sigs, NULL);
 
-    /* watch atp and dsi sockets. */
-    FD_ZERO(&save_rfds);
-    for (config = configs; config; config = config->next) {
-        if (config->fd < 0) /* for proxies */
-            continue;
-        FD_SET(config->fd, &save_rfds);
+    /* watch atp, dsi sockets and ipc parent/child file descriptor. */
+    if ((ipc = server_ipc_create())) {
+        Ipc_fd = server_ipc_parent(ipc);
     }
+    set_fd(Ipc_fd);
 
     /* wait for an appleshare connection. parent remains in the loop
      * while the children get handled by afp_over_{asp,dsi}.  this is
@@ -259,12 +307,15 @@ char	**av;
             LOG(log_error, logtype_afpd, "main: can't wait for input: %s", strerror(errno));
             break;
         }
-
+        if (Ipc_fd >=0 && FD_ISSET(Ipc_fd, &rfds)) {
+            server_ipc_read(server_children);
+        }
         for (config = configs; config; config = config->next) {
             if (config->fd < 0)
                 continue;
-            if (FD_ISSET(config->fd, &rfds))
+            if (FD_ISSET(config->fd, &rfds)) {
                 config->server_start(config, configs, server_children);
+            }
         }
     }
 
