@@ -1,5 +1,5 @@
 /* 
- * $Id: mangle.c,v 1.16.2.1.2.12 2004/09/02 20:48:52 didg Exp $ 
+ * $Id: mangle.c,v 1.16.2.1.2.12.2.3 2005/02/14 16:01:54 didg Exp $ 
  *
  * Copyright (c) 2002. Joe Marcus Clarke (marcus@marcuscom.com)
  * All Rights Reserved.  See COPYRIGHT.
@@ -54,8 +54,9 @@ static char *demangle_checks ( const struct vol *vol, char* uname, char * mfilen
 				      uname, strlen(uname), buffer, MAXPATHLEN, &flags)) ) {
 	return mfilename;
     }
-    /* If the filename is too long we also needed to mangle isn't this stuff always false */
-    if ( len >= vol->max_filename || (mfilenamelen = strlen(mfilename)) == MACFILELEN ) {
+    /* If the filename is too long we also needed to mangle */
+    mfilenamelen = strlen(mfilename);
+    if ( len >= vol->max_filename || mfilenamelen == MACFILELEN ) {
         flags |= CONV_REQMANGLE;
         len = prefix;
     }
@@ -205,8 +206,8 @@ demangle_osx(const struct vol *vol, char *mfilename, cnid_t did, cnid_t *fileid)
 /* -------------------------------------------------------
  * find the start of a utf8 character
  */
-static char *
-utf8_mangle_validate(char *path, size_t len)
+static unsigned char *
+utf8_mangle_validate(unsigned char *path, size_t len)
 {
     unsigned char *p = path + len;
     int           dec = 0;
@@ -230,12 +231,12 @@ utf8_mangle_validate(char *path, size_t len)
    id         file/folder ID or 0
    
 */
-char *
-mangle(const struct vol *vol, char *filename, size_t filenamelen, char *uname, cnid_t id, int flags) {
-    char *ext = NULL;
-    char *m = NULL;
-    static char mfilename[MAXPATHLEN + 1];
-    char mangle_suffix[MANGLE_LENGTH + 1];
+unsigned char *
+mangle(const struct vol *vol, unsigned char *filename, size_t filenamelen, unsigned char *uname, cnid_t id, int flags) {
+    unsigned char *ext = NULL;
+    unsigned char *m = NULL;
+    static unsigned char mfilename[MAXPATHLEN + 1];
+    unsigned char mangle_suffix[MANGLE_LENGTH + 1];
     size_t ext_len = 0;
     size_t maxlen;
     int k;
@@ -245,6 +246,11 @@ mangle(const struct vol *vol, char *filename, size_t filenamelen, char *uname, c
     if (!(flags & 1) && filenamelen <= maxlen) {
 	return filename;
     }
+
+    if (!id) {
+        /* we don't have the file id! only catsearch call mangle with id == 0 */
+        return NULL;
+    }
     /* First, attempt to locate a file extension. */
     if (NULL != (ext = strrchr(uname, '.')) ) {
 	ext_len = strlen(ext);
