@@ -1,5 +1,5 @@
 /*
- * $Id: uam.c,v 1.24.6.7 2004/10/08 00:16:09 bfernhomberg Exp $
+ * $Id: uam.c,v 1.24.6.7.2.3 2005/02/01 11:33:48 didg Exp $
  *
  * Copyright (c) 1999 Adrian Sun (asun@zoology.washington.edu)
  * All Rights Reserved.  See COPYRIGHT.
@@ -372,9 +372,10 @@ int uam_checkuser(const struct passwd *pwd)
 int uam_random_string (AFPObj *obj, char *buf, int len)
 {
     u_int32_t result;
+    int ret;
     int fd;
 
-    if ( (len < 0) || (len % sizeof(result)))
+    if ( (len <= 0) || (len % sizeof(result)))
             return -1;
 
     /* construct a random number */
@@ -391,9 +392,9 @@ int uam_random_string (AFPObj *obj, char *buf, int len)
             memcpy(buf + i, &result, sizeof(result));
         }
     } else {
-        result = read(fd, buf, len);
+        ret = read(fd, buf, len);
         close(fd);
-        if (result < 0)
+        if (ret <= 0)
             return -1;
     }
     return 0;
@@ -406,20 +407,19 @@ int uam_afpserver_option(void *private, const int what, void *option,
 AFPObj *obj = private;
     char **buf = (char **) option; /* most of the options are this */
     struct session_info **sinfo = (struct session_info **) option;
-    int32_t result;
 
     if (!obj || !option)
         return -1;
 
     switch (what) {
     case UAM_OPTION_USERNAME:
-        *buf = (void *) obj->username;
+        *buf = obj->username;
         if (len)
             *len = sizeof(obj->username) - 1;
         break;
 
     case UAM_OPTION_GUEST:
-        *buf = (void *) obj->options.guest;
+        *buf = obj->options.guest;
         if (len)
             *len = strlen(obj->options.guest);
         break;
@@ -430,7 +430,7 @@ AFPObj *obj = private;
 
         switch (*len) {
         case UAM_PASSWD_FILENAME:
-            *buf = (void *) obj->options.passwdfile;
+            *buf = obj->options.passwdfile;
             *len = strlen(obj->options.passwdfile);
             break;
 
@@ -458,21 +458,22 @@ AFPObj *obj = private;
         break;
 
     case UAM_OPTION_RANDNUM: /* returns a random number in 4-byte units. */
-        if (!len || (*len < 0) || (*len % sizeof(result)))
+        if (!len)
             return -1;
 
-        return uam_random_string(obj, *buf, *len);
+        return uam_random_string(obj, option, *len);
         break;
 
     case UAM_OPTION_HOSTNAME:
-        *buf = (void *) obj->options.hostname;
+        *buf = obj->options.hostname;
         if (len)
             *len = strlen(obj->options.hostname);
         break;
 
     case UAM_OPTION_PROTOCOL:
-        *buf = (void *) obj->proto;
+        *((int *) option) = obj->proto;
         break;
+        
     case UAM_OPTION_CLIENTNAME:
         {
             struct DSI *dsi = obj->handle;
@@ -482,9 +483,9 @@ AFPObj *obj = private;
                                 sizeof( struct in_addr ),
                                 dsi->client.sin_family );
             if( hp )
-                *buf = (void *) hp->h_name;
+                *buf = hp->h_name;
             else
-                *buf = (void *) inet_ntoa( dsi->client.sin_addr );
+                *buf = inet_ntoa( dsi->client.sin_addr );
         }
         break;
     case UAM_OPTION_COOKIE:
@@ -499,6 +500,16 @@ AFPObj *obj = private;
         if (len)
             *len = (*buf)?strlen(*buf):0;
 	break;
+    case UAM_OPTION_KRB5REALM:
+	*buf = obj->options.k5realm;
+        if (len)
+            *len = (*buf)?strlen(*buf):0;
+	break;
+    case UAM_OPTION_FQDN:
+	*buf = obj->options.fqdn;
+        if (len)
+            *len = (*buf)?strlen(*buf):0;
+	break;
     case UAM_OPTION_MACCHARSET:
         *((int *) option) = obj->options.maccharset;
         *len = sizeof(obj->options.maccharset);
