@@ -1,5 +1,5 @@
 /*
- * $Id: config.c,v 1.13.6.5 2004/06/09 01:10:07 bfernhomberg Exp $
+ * $Id: config.c,v 1.13.6.5.2.3 2005/02/06 10:16:02 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
@@ -69,7 +69,7 @@ char *strchr (), *strrchr ();
 
 int	router(), dontroute(), seed(), phase(), net(), addr(), zone(), noallmulti();
 
-static struct param {
+static const struct param {
     char	*p_name;
     int		(*p_func)();
 } params[] = {
@@ -276,7 +276,7 @@ int writeconf( cf )
                     }
                     len = ((struct ziptab *)l->l_data)->zt_len;
                 } 
-		fprintf( newconf, " -zone \"%.*s\"", len, zonename);
+		fprintf( newconf, " -zone \"%.*s\"", (int)len, zonename);
                 free(zonename);
 	    }
 	    fprintf( newconf, "\n" );
@@ -328,7 +328,7 @@ int readconf( cf )
     struct interface	*iface, *niface;
     char		line[ MAXLINELEN ], **argv, *p;
     unsigned int	i, j;
-    int			s, cc;
+    int			s, cc = 0;
     FILE		*conf;
 
     if ( cf == NULL ) {
