@@ -1,5 +1,5 @@
 /*
- * $Id: uams_dhx_passwd.c,v 1.16.2.1 2003/06/14 17:12:17 srittau Exp $
+ * $Id: uams_dhx_passwd.c,v 1.18.6.6 2004/03/18 02:56:32 bfernhomberg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * Copyright (c) 1999 Adrian Sun (asun@u.washington.edu) 
@@ -10,23 +10,35 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#ifdef NETBSD
+#define _XOPEN_SOURCE 500 /* for crypt() */
+#endif
+#ifdef FREEBSD
+#define _XOPEN_SOURCE /* for crypt() */
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
-#ifndef NO_CRYPT_H
+#ifdef HAVE_CRYPT_H
 #include <crypt.h>
-#endif /* ! NO_CRYPT_H */
+#endif /* ! HAVE_CRYPT_H */
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #include <pwd.h>
-#include <atalk/logger.h>
-
 #ifdef SHADOWPW
 #include <shadow.h>
 #endif /* SHADOWPW */
-
-#ifdef OPENSSL_DHX
+#if defined(GNUTLS_DHX)
+#include <gnutls/openssl.h>
+#elif defined(OPENSSL_DHX)
 #include <openssl/bn.h>
 #include <openssl/dh.h>
 #include <openssl/cast.h>
@@ -36,6 +48,7 @@
 #include <cast.h>
 #endif /* OPENSSL_DHX */
 
+#include <atalk/logger.h>
 #include <atalk/afp.h>
 #include <atalk/uam.h>
 
@@ -61,7 +74,7 @@ static char *clientname;
 #endif /* TRU64 */
 
 /* dhx passwd */
-static int passwd_login(void *obj, struct passwd **uam_pwd,
+static int pwd_login(void *obj, char *username, int ulen, struct passwd **uam_pwd,
 			char *ibuf, int ibuflen,
 			char *rbuf, int *rbuflen)
 {
@@ -74,8 +87,7 @@ static int passwd_login(void *obj, struct passwd **uam_pwd,
 #endif /* SHADOWPW */
     BIGNUM *bn, *gbn, *pbn;
     u_int16_t sessid;
-    int len, i;
-    char *name;
+    int i;
     DH *dh;
 
 #ifdef TRU64
@@ -87,37 +99,23 @@ static int passwd_login(void *obj, struct passwd **uam_pwd,
 
     *rbuflen = 0;
 
-    if (uam_afpserver_option(obj, UAM_OPTION_USERNAME, (void *) &name, &i) < 0)
-      return AFPERR_PARAM;
-
 #ifdef TRU64
     if( uam_afpserver_option( obj, UAM_OPTION_CLIENTNAME,
                               (void *) &clientname, NULL ) < 0 )
         return AFPERR_PARAM;
 #endif /* TRU64 */
 
-    len = (unsigned char) *ibuf++;
-    if ( len + 1 > i ) {
-	return( AFPERR_PARAM );
+    if (( dhxpwd = uam_getname(obj, username, ulen)) == NULL ) {
+	return AFPERR_PARAM;
     }
-
-    memcpy(name, ibuf, len );
-    ibuf += len;
-    name[ len ] = '\0';
-    if ((unsigned long) ibuf & 1) /* padding */
-      ++ibuf;
-
-    if (( dhxpwd = uam_getname(name, i)) == NULL ) {
-      return AFPERR_PARAM;
-    }
-
-    LOG(log_info, logtype_uams, "dhx login: %s", name);
+    
+    LOG(log_info, logtype_uams, "dhx login: %s", username);
     if (uam_checkuser(dhxpwd) < 0)
       return AFPERR_NOTAUTH;
 
 #ifdef SHADOWPW
     if (( sp = getspnam( dhxpwd->pw_name )) == NULL ) {
-	LOG(log_info, logtype_uams, "no shadow passwd entry for %s", name);
+	LOG(log_info, logtype_uams, "no shadow passwd entry for %s", username);
 	return AFPERR_NOTAUTH;
     }
     dhxpwd->pw_passwd = sp->sp_pwdp;
@@ -209,14 +207,87 @@ passwd_fail:
     return AFPERR_PARAM;
 }
 
+/* cleartxt login */
+static int passwd_login(void *obj, struct passwd **uam_pwd,
+			char *ibuf, int ibuflen,
+			char *rbuf, int *rbuflen)
+{
+    char *username;
+    int len, ulen;
+
+    *rbuflen = 0;
+
+    if (uam_afpserver_option(obj, UAM_OPTION_USERNAME,
+			     (void *) &username, &ulen) < 0)
+	return AFPERR_MISC;
+
+    if (ibuflen <= 1) {
+	return( AFPERR_PARAM );
+    }
+
+    len = (unsigned char) *ibuf++;
+    ibuflen--;
+    if (!len || len > ibuflen || len > ulen ) {
+	return( AFPERR_PARAM );
+    }
+    memcpy(username, ibuf, len );
+    ibuf += len;
+    ibuflen -=len;
+    username[ len ] = '\0';
+
+    if ((unsigned long) ibuf & 1) { /* pad character */
+	++ibuf;
+	ibuflen--;
+    }
+    return (pwd_login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen));
+    
+}
+
+/* cleartxt login ext 
+ * uname format :
+    byte      3
+    2 bytes   len (network order)
+    len bytes utf8 name
+*/
+static int passwd_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
+			char *ibuf, int ibuflen,
+			char *rbuf, int *rbuflen)
+{
+    char       *username;
+    int        len, ulen;
+    u_int16_t  temp16;
+
+    *rbuflen = 0;
+    
+    if (uam_afpserver_option(obj, UAM_OPTION_USERNAME,
+			     (void *) &username, &ulen) < 0)
+	return AFPERR_MISC;
+
+    if (*uname != 3)
+	return AFPERR_PARAM;
+    uname++;
+    memcpy(&temp16, uname, sizeof(temp16));
+    len = ntohs(temp16);
+    if (!len || len > ulen ) {
+	return( AFPERR_PARAM );
+    }
+    memcpy(username, uname +2, len );
+    username[ len ] = '\0';
+    return (pwd_login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen));
+}
+			
 static int passwd_logincont(void *obj, struct passwd **uam_pwd,
 			    char *ibuf, int ibuflen, 
 			    char *rbuf, int *rbuflen)
 {
+#ifdef SHADOWPW
+    struct spwd *sp;
+#endif /* SHADOWPW */
     unsigned char iv[] = "LWallace";
     BIGNUM *bn1, *bn2, *bn3;
     u_int16_t sessid;
     char *p;
+    int err = AFPERR_NOTAUTH;
 
     *rbuflen = 0;
 
@@ -285,8 +356,25 @@ static int passwd_logincont(void *obj, struct passwd **uam_pwd,
     memset(rbuf, 0, PASSWDLEN);
     if ( strcmp( p, dhxpwd->pw_passwd ) == 0 ) {
       *uam_pwd = dhxpwd;
-      return AFP_OK;
+      err = AFP_OK;
     }
+#ifdef SHADOWPW
+    if (( sp = getspnam( dhxpwd->pw_name )) == NULL ) {
+	LOG(log_info, logtype_uams, "no shadow passwd entry for %s", dhxpwd->pw_name);
+	return (AFPERR_NOTAUTH);
+    }
+
+    /* check for expired password */
+    if (sp && sp->sp_max != -1 && sp->sp_lstchg) {
+        time_t now = time(NULL) / (60*60*24);
+        int32_t expire_days = sp->sp_lstchg - now + sp->sp_max;
+        if ( expire_days < 0 ) {
+                LOG(log_info, logtype_uams, "password for user %s expired", dhxpwd->pw_name);
+		err = AFPERR_PWDEXPR;
+        }
+    }
+#endif /* SHADOWPW */
+    return err;
 #endif /* TRU64 */
 
     return AFPERR_NOTAUTH;
@@ -295,8 +383,8 @@ static int passwd_logincont(void *obj, struct passwd **uam_pwd,
 
 static int uam_setup(const char *path)
 {
-  if (uam_register(UAM_SERVER_LOGIN, path, "DHCAST128",
-		   passwd_login, passwd_logincont, NULL) < 0)
+  if (uam_register(UAM_SERVER_LOGIN_EXT, path, "DHCAST128",
+		   passwd_login, passwd_logincont, NULL, passwd_login_ext) < 0)
     return -1;
   /*uam_register(UAM_SERVER_PRINTAUTH, path, "DHCAST128",
     passwd_printer);*/
