@@ -1,5 +1,5 @@
 /*
- * $Id: ad_private.h,v 1.3.2.1 2003/11/20 17:21:01 bfernhomberg Exp $
+ * $Id: ad_private.h,v 1.5 2003/01/31 11:26:36 didg Exp $
  */
 
 #ifndef LIBATALK_ADOUBLE_AD_PRIVATE_H
@@ -21,7 +21,7 @@
 
 #define adf_lock_init(a) do { \
 	(a)->adf_lockmax = (a)->adf_lockcount = 0; \
-	(a)->adf_lock = NULL; \
+	(a)->adf_excl = 0;(a)->adf_lock = NULL; \
 } while (0)
 
 #define adf_lock_free(a) do { \
